body {
  padding-top: 60px !important;
  background-image: url('assets/images/logo.png');
  background-repeat: no-repeat;
  background-attachment:fixed;
  background-position: center center;
  min-height: 800px;
}

